"""
Python module to handle control of the Kodiak modules.

This class will also do all necessary interaction with underlying Database.

Author : Matt Holsey
Date : 09-05-2023
"""
import logging
import time
import os
import paramiko
import threading
from .sb_db import SimpleDB

from .kodiak_session_management import KodiakSession
from .KodiakGlobals import KodiakGlobals
from .kodiak_endpoints import *


"""
API KEY:
Name : mholsey
desc : mholsey test key
ID   : 

API Key : gThV9h4LQmmSiYFNTcjBuvoRPvEC3LpW
# The key is a 'secret key' so it would have to be generated once by the suer
"""


class KodiakControlClass:

    def __init__(self):

        # self.private_key = private_key if private_key else "gThV9h4LQmmSiYFNTcjBuvoRPvEC3LpW"
        self.db = SimpleDB(KodiakGlobals.database_name)

        # If the table doesn't already exist, then create a new one
        if not self.db.check_exists():
            self.db.create_table()

        self.kodiak_session = KodiakSession()

    def login(self, kw):
        """
        Login to the Kodiak

        Will attempt to log in and return the response received.

        If login was successful, it will also save these details to the database.

        :param kw: KodiakWrapper object containing the user's credentials

        :return: Boolean : True if the login worked, else False
        """

        # Attempt to log in
        response = self.kodiak_session.login(kw)

        # If there's nothing returned, then there was an error logging into the system
        if response:
            # Add the new session key
            self.db.update_kodiak_db_session_key(kw.ipaddress, response['session_token'], response['refresh_token'])
            return True
        else:
            return False

    # def refresh(self, kw):
    #     """
    #     Attempt to refresh the module's login.
    #
    #     :param kw: Kodiak wrapper containing credentials
    #
    #     :return: boolean: True if success, else False
    #     """
    #
    #     # Check if the kodiak module present in database
    #     response = self.db.get_kodiak_db(kw.ipaddress)
    #     if response:
    #         # Attempt to refresh the module's login
    #         refresh_response = self.kodiak_session.refresh_login(kw, response[6], response[3])
    #         if refresh_response:
    #             # Update the database with the new refresh / session tokens
    #             self.db.update_kodiak_db_session_key(kw.ipaddress, refresh_response['session_key'],
    #                                                  refresh_response['refresh_token'])
    #             return True
    #     else:
    #         return False

    def check_user_exists(self, kw):
        """
        Function to get a list of kodiak users and check if the standard SB one exists

        :param kw: Kodiak Wrapper
        :return: boolean - True if the sb user exists, else False
        """

        # Getting the list of users from /users
        users = self.kodiak_session.get_users(kw)
        # Standard sb user is in globals, check if string in string
        if KodiakGlobals.my_kodiak_user in str(users):
            return True

        return False

    def create_user(self, kw):
        """
        Checking and creating users on the Kodiak Module.
        This will get the user and update the database on this system with that user

        :param kw: Kodiak Wrapper

        :return: boolean : True if success, else False.
        """

        # Check if Module in DB
        db_response = self.db.get_kodiak_db(kw.ipaddress)
        if db_response:
            # check if there`s a user assigned to the kodiak, if not, create one
            if not db_response[1]:
                # if not self.check_user_exists(kw):
                created_user = self.kodiak_session.post_add_users(kw)
                print(created_user)

                self.db.update_kodiak_db_kodiak_user(kw.ipaddress, KodiakGlobals.my_kodiak_user)
                self.db.update_kodiak_db_kodiak_password(kw.ipaddress, KodiakGlobals.my_kodiak_pass)

            return True

        else:
            # return False if we couldn`t create one
            return False

    def lock(self, kw, lock_name):
        """
        Attempts to lock the Kodiak Module

        :param kw: Kodiak Wrapper
        :param lock_name: Lock name to be used for lock

        :return: boolean : True if success, else False.
        """

        # Check if Module in DB
        db_response = self.db.get_kodiak_db(kw.ipaddress)
        if db_response:
            # Attempt to lock the module
            # lock_response = self.kodiak_session.lock(kw, lock_name, db_response[4])
            lock_response = self.kodiak_session.lock(kw, lock_name, None)
            if lock_response:
                # Use the 'lock_key' response and update the DB with the key.
                self.db.update_kodiak_db_lock_key(kw.ipaddress, lock_response)
                return True
        else:

            return False

    def unlock(self, kw):
        """
        Attempt to unlock the module.

        :param kw: Kodiak wrapper containing credentials

        :return: boolean: True if success, else False
        """

        # Check if the kodiak module present in database
        response = self.db.get_kodiak_db(kw.ipaddress)
        if response:
            # Attempt to unlock the module based off of the current items in the DB
            if self.kodiak_session.unlock(kw, response[-1], response[4]):
                return True

        else:
            return False

    def get_lock_status(self, kw):
        """
        Function to get the current lock status of the kodiak module

        :param kw: Kodiak Wrapper containing credentials

        :return: Lock status or None if no response
        """
        response = self.db.get_kodiak_db(kw.ipaddress)
        if response:
            # Attempt to refresh the module's login
            lock_status = self.kodiak_session.get_lock_status(kw, response[4])
            return lock_status
        else:
            return None

    def stop(self, kw):
        """
        Sending a POST request to stop the Kodiak capture

        :return: True if success, False if fail
        """

        query_response = self.db.get_kodiak_db(kw.ipaddress)
        if not query_response:
            logging.warning("Couldn't find kodiak login in database, have you logged into the module yet?")
            return False

        message_to_send = {
            "lock_key": query_response[-1]
        }
        headers = {
            'x-api-key': kw.key
        }

        endpoint = KodiakGlobals.url_prefix + kw.ipaddress + endpoint_stop

        # A post request to the API
        response = self.kodiak_session.send_post_command(message_to_send, endpoint, headers)

        if 'error' in response:
            return False

        return True

    def start(self, kw, trigger=False):
        """
        Sending a POST request to start the kodiak capture

        :return: Return True if start succeeded, else False
        """

        query_response = self.db.get_kodiak_db(kw.ipaddress)
        if not query_response:
            logging.warning("Couldn't find kodiak login in database, have you logged into the module yet?")
            return False

        # message_to_send = {
        #     "channels": [
        #         {
        #             "name": "dn.0",
        #             "type": "data",
        #             "capture": {
        #               "buffer": {
        #                 "limit": 77309411328,
        #                 "post_trigger": 0,
        #                 "trigger_position": 100
        #               },
        #               "lane_control": None,
        #               "speed_control": "auto",
        #               "transceiver_control": {
        #                 "equalisation": "auto",
        #                 "sris": False
        #               },
        #               "tlp_truncation": 0
        #             },
        #             "prefilter": {
        #               "dllp": {
        #                 "initfc1": True,
        #                 "initfc2": True,
        #                 "updatefc": True
        #               },
        #               "os": {
        #                 "skp": True,
        #                 "cskp": True
        #               }
        #             }
        #         },
        #         {
        #             "type": "data",
        #             "name": "up.0",
        #             "capture": {
        #                 "buffer": {
        #                     "limit": 77309411328,
        #                     "post_trigger": 0,
        #                     "trigger_position": 100
        #                 },
        #                 "lane_control": {
        #                     "assignment": {
        #                         "mode": "manual",
        #                         "lanes": [
        #                             0,
        #                             1,
        #                             3,
        #                             2,
        #                             4,
        #                             5,
        #                             6,
        #                             7,
        #                             8,
        #                             9,
        #                             10,
        #                             11,
        #                             12,
        #                             13,
        #                             14,
        #                             15
        #                         ]
        #                     },
        #                     "lane_polarity": [
        #                         None,
        #                         None,
        #                         None,
        #                         None,
        #                         True,
        #                         True,
        #                         True,
        #                         True,
        #                         False,
        #                         False,
        #                         False,
        #                         False,
        #                         None,
        #                         None,
        #                         None,
        #                         None
        #                     ]
        #                 },
        #                 "speed_control": "gen5",
        #                 "transceiver_control": {
        #                     "equalisation": "dfe",
        #                     "sris": True
        #                 },
        #                 "tlp_truncation": 0
        #             },
        #             "prefilter": {
        #                 "dllp": {
        #                     "initfc1": True,
        #                     "initfc2": True,
        #                     "updatefc": True
        #                 },
        #                 "os": {
        #                     "skp": True,
        #                     "cskp": True
        #                 }
        #             }
        #         }
        #     ],
        #
        #     "mode": "trigger" if trigger else "manual",
        #     "lock_key": query_response[-1],
        #     # "lock_key": KodiakGlobals.lock_name,
        # }

        message_to_send = {"channels":[{"capture":{"buffer":{"limit":7730941104,"trigger_position":100},"strong_compression":False,"tlp_truncation":0},"name":"dn.0","prefilter":{"dllp":{"ack":True,"nack":True,"pm_enter_l1":True,"pm_enter_l23":True,"pm_request_l1":True,"pm_request_ack":True,"vendor":True,"mrinit":True,"dlf":True,"nop":True,"mrinit_fc1":True,"mrinit_fc2":True,"mrupdate_fc":True,"initfc1_p":True,"initfc1_np":True,"initfc1_cpl":True,"initfc2_p":True,"initfc2_np":True,"initfc2_cpl":True,"updatefc_p":True,"updatefc_np":True,"updatefc_cpl":True},"os":{"ack":True,"nack":True,"pm_enter_l1":True,"pm_enter_l23":True,"pm_request_l1":True,"pm_request_ack":True,"vendor":True,"mrinit":True,"dlf":True,"nop":True,"mrinit_fc1":True,"mrinit_fc2":True,"mrupdate_fc":True,"initfc1_p":True,"initfc1_np":True,"initfc1_cpl":True,"initfc2_p":True,"initfc2_np":True,"initfc2_cpl":True,"updatefc_p":True,"updatefc_np":True,"updatefc_cpl":True,"skp":True,"cskp":True,"fts":True,"eios":True,"eieos":True,"sds":True,"eds":True,"ts1":True,"ts2":True},"error":{"ack":True,"nack":True,"pm_enter_l1":True,"pm_enter_l23":True,"pm_request_l1":True,"pm_request_ack":True,"vendor":True,"mrinit":True,"dlf":True,"nop":True,"mrinit_fc1":True,"mrinit_fc2":True,"mrupdate_fc":True,"initfc1_p":True,"initfc1_np":True,"initfc1_cpl":True,"initfc2_p":True,"initfc2_np":True,"initfc2_cpl":True,"updatefc_p":True,"updatefc_np":True,"updatefc_cpl":True,"skp":True,"cskp":True,"fts":True,"eios":True,"eieos":True,"sds":True,"eds":True,"ts1":True,"ts2":True,"8b_10b":True,"128b_130b":True},"cxl":{"ack":True,"nack":True,"pm_enter_l1":True,"pm_enter_l23":True,"pm_request_l1":True,"pm_request_ack":True,"vendor":True,"mrinit":True,"dlf":True,"nop":True,"mrinit_fc1":True,"mrinit_fc2":True,"mrupdate_fc":True,"initfc1_p":True,"initfc1_np":True,"initfc1_cpl":True,"initfc2_p":True,"initfc2_np":True,"initfc2_cpl":True,"updatefc_p":True,"updatefc_np":True,"updatefc_cpl":True,"skp":True,"cskp":True,"fts":True,"eios":True,"eieos":True,"sds":True,"eds":True,"ts1":True,"ts2":True,"8b_10b":True,"128b_130b":True,"io":True,"io_ieds":True,"cachemem":True,"cachemem_ieds":True,"null_ieds":True,"almp":True,"almp_ieds":True,"reserved":True},"cachemem":{"ack":True,"nack":True,"pm_enter_l1":True,"pm_enter_l23":True,"pm_request_l1":True,"pm_request_ack":True,"vendor":True,"mrinit":True,"dlf":True,"nop":True,"mrinit_fc1":True,"mrinit_fc2":True,"mrupdate_fc":True,"initfc1_p":True,"initfc1_np":True,"initfc1_cpl":True,"initfc2_p":True,"initfc2_np":True,"initfc2_cpl":True,"updatefc_p":True,"updatefc_np":True,"updatefc_cpl":True,"skp":True,"cskp":True,"fts":True,"eios":True,"eieos":True,"sds":True,"eds":True,"ts1":True,"ts2":True,"8b_10b":True,"128b_130b":True,"io":True,"io_ieds":True,"cachemem":True,"cachemem_ieds":True,"null_ieds":True,"almp":True,"almp_ieds":True,"reserved":True,"data_be":True,"llcrd_ack":True,"retry_idle":True,"retry_req":True,"retry_ack":True,"retry_frame":True,"ide_idle":True,"ide_start":True,"ide_tmac":True,"init_param":True}},"type":"data"},{"capture":{"buffer":{"limit":7730941104,"trigger_position":100},"strong_compression":False,"tlp_truncation":0},"name":"up.0","prefilter":{"dllp":{"ack":True,"nack":True,"pm_enter_l1":True,"pm_enter_l23":True,"pm_request_l1":True,"pm_request_ack":True,"vendor":True,"mrinit":True,"dlf":True,"nop":True,"mrinit_fc1":True,"mrinit_fc2":True,"mrupdate_fc":True,"initfc1_p":True,"initfc1_np":True,"initfc1_cpl":True,"initfc2_p":True,"initfc2_np":True,"initfc2_cpl":True,"updatefc_p":True,"updatefc_np":True,"updatefc_cpl":True},"os":{"ack":True,"nack":True,"pm_enter_l1":True,"pm_enter_l23":True,"pm_request_l1":True,"pm_request_ack":True,"vendor":True,"mrinit":True,"dlf":True,"nop":True,"mrinit_fc1":True,"mrinit_fc2":True,"mrupdate_fc":True,"initfc1_p":True,"initfc1_np":True,"initfc1_cpl":True,"initfc2_p":True,"initfc2_np":True,"initfc2_cpl":True,"updatefc_p":True,"updatefc_np":True,"updatefc_cpl":True,"skp":True,"cskp":True,"fts":True,"eios":True,"eieos":True,"sds":True,"eds":True,"ts1":True,"ts2":True},"error":{"ack":True,"nack":True,"pm_enter_l1":True,"pm_enter_l23":True,"pm_request_l1":True,"pm_request_ack":True,"vendor":True,"mrinit":True,"dlf":True,"nop":True,"mrinit_fc1":True,"mrinit_fc2":True,"mrupdate_fc":True,"initfc1_p":True,"initfc1_np":True,"initfc1_cpl":True,"initfc2_p":True,"initfc2_np":True,"initfc2_cpl":True,"updatefc_p":True,"updatefc_np":True,"updatefc_cpl":True,"skp":True,"cskp":True,"fts":True,"eios":True,"eieos":True,"sds":True,"eds":True,"ts1":True,"ts2":True,"8b_10b":True,"128b_130b":True},"cxl":{"ack":True,"nack":True,"pm_enter_l1":True,"pm_enter_l23":True,"pm_request_l1":True,"pm_request_ack":True,"vendor":True,"mrinit":True,"dlf":True,"nop":True,"mrinit_fc1":True,"mrinit_fc2":True,"mrupdate_fc":True,"initfc1_p":True,"initfc1_np":True,"initfc1_cpl":True,"initfc2_p":True,"initfc2_np":True,"initfc2_cpl":True,"updatefc_p":True,"updatefc_np":True,"updatefc_cpl":True,"skp":True,"cskp":True,"fts":True,"eios":True,"eieos":True,"sds":True,"eds":True,"ts1":True,"ts2":True,"8b_10b":True,"128b_130b":True,"io":True,"io_ieds":True,"cachemem":True,"cachemem_ieds":True,"null_ieds":True,"almp":True,"almp_ieds":True,"reserved":True},"cachemem":{"ack":True,"nack":True,"pm_enter_l1":True,"pm_enter_l23":True,"pm_request_l1":True,"pm_request_ack":True,"vendor":True,"mrinit":True,"dlf":True,"nop":True,"mrinit_fc1":True,"mrinit_fc2":True,"mrupdate_fc":True,"initfc1_p":True,"initfc1_np":True,"initfc1_cpl":True,"initfc2_p":True,"initfc2_np":True,"initfc2_cpl":True,"updatefc_p":True,"updatefc_np":True,"updatefc_cpl":True,"skp":True,"cskp":True,"fts":True,"eios":True,"eieos":True,"sds":True,"eds":True,"ts1":True,"ts2":True,"8b_10b":True,"128b_130b":True,"io":True,"io_ieds":True,"cachemem":True,"cachemem_ieds":True,"null_ieds":True,"almp":True,"almp_ieds":True,"reserved":True,"data_be":True,"llcrd_ack":True,"retry_idle":True,"retry_req":True,"retry_ack":True,"retry_frame":True,"ide_idle":True,"ide_start":True,"ide_tmac":True,"init_param":True}},"type":"data"},{"capture":{},"name":"sideband.0","prefilter":{"0":True,"1":True,"4":True,"5":True,"6":True,"7":True,"8":True,"9":True,"10":True,"11":True,"12":True,"13":True,"17":True,"30":True,"31":True},"type":"sideband"},{"enable":False,"name":"smbus.0","prefilter":{"addresses":[],"nacks":False},"type":"smbus"}],"mode":"trigger","recording":False,"trigger":[{"resources":{"counters":[],"timers":[]},"start":["State 0"],"states":[{"conditions":[{"fields":[{"mask":255,"match":68,"offset":0},{"mask":15,"match":1,"offset":3},{"mask":240,"match":0,"offset":7},{"mask":15,"match":0,"offset":10},{"mask":252,"match":0,"offset":11},{"mask":255,"match":227,"offset":12},{"mask":255,"match":26,"offset":13}],"type":"tlp_data"}],"name":"State 0","onMatch":[{"type":"trigger"}]}],"type":"data","link":0}],
                           "mode": "trigger" if trigger else "manual",
                           "lock_key": query_response[-1],
                           }

        # If we want to trigger the kodiak module, then add the SB_trigger definition in.
        if trigger:
            message_to_send['trigger'] = {
                "resources": {
                    "counters": [],
                    "timers": []
                },
                "start": [
                    "State 0"
                ],
                "states": [
                    {
                        "conditions": [
                            {
                                "fields": [
                                    {
                                        "mask": 255,
                                        "match": 68,
                                        "offset": 0
                                    },
                                    {
                                        "mask": 15,
                                        "match": 1,
                                        "offset": 3
                                    },
                                    {
                                        "mask": 240,
                                        "match": 0,
                                        "offset": 7
                                    },
                                    {
                                        "mask": 15,
                                        "match": 0,
                                        "offset": 10
                                    },
                                    {
                                        "mask": 252,
                                        "match": 0,
                                        "offset": 11
                                    },
                                    {
                                        "mask": 255,
                                        "match": 227,
                                        "offset": 12
                                    },
                                    {
                                        "mask": 255,
                                        "match": 26,
                                        "offset": 13
                                    }
                                ],
                                "type": "tlp_data"
                            }
                        ],
                        "name": "State 0",
                        "onMatch": [
                            {
                                "type": "trigger"
                            }
                        ]
                    }
                ]
            }

        headers = {
            'x-api-key': kw.key
        }

        endpoint = KodiakGlobals.url_prefix + kw.ipaddress + endpoint_start

        # A post request to the API
        response = self.kodiak_session.send_post_command(message_to_send, endpoint, headers)

        if 'error' in response:
            return False

        return True

    def get_module_status(self, kw):
        """
        Send a GET request to the Kodiak to see its current lock status.

        :return: Response from get lock stats request
        """

        query_response = self.db.get_kodiak_db(kw.ipaddress)

        headers = {
            'x-api-key': kw.key
        }

        endpoint = KodiakGlobals.url_prefix + kw.ipaddress + endpoint_status

        # Do the get request to lock endpoint
        response = self.kodiak_session.send_get_request(endpoint, headers)

        return response

    def get_module_capture_status(self, kw):
        """
        Send a GET request to the Kodiak to see its current lock status.

        :return: Response from get lock stats request
        """

        query_response = self.db.get_kodiak_db(kw.ipaddress)

        headers = {
            'x-api-key': kw.key
        }

        endpoint = KodiakGlobals.url_prefix + kw.ipaddress + endpoint_capture_status

        # Do the get request to lock endpoint
        response = self.kodiak_session.send_get_request(endpoint, headers)

        return response

    def locate_kodiak_port(self, kw, remote_ip=None):
        """
        Go through and trigger the SB system to trigger the Analyser that should be running.
        :return:
        """

        ssh = None

        # If there's a remote IP specified, use this, else just execute the command on the current system.
        if remote_ip:
            ssh = paramiko.SSHClient()
            ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
            ssh.connect(remote_ip, 22, username='root', password='sb')

        for x in range(16):
            index = x
            if len(str(index)) < 2:
                index = "0" + str(index)

            os_cmd = f'sbecho trigger=1{index} > /proc/vlun/nvme'

            if remote_ip:
                ssh_stdin, ssh_stdout, ssh_stderr = ssh.exec_command(os_cmd)
                print(ssh_stdin, ssh_stdout, ssh_stderr)
            else:
                os.system(os_cmd)

            time.sleep(0.05)

            response = self.get_module_capture_status(kw)

            if response:
                if 'error' not in response:

                    if response[0]['triggered']:
                        # Adding the port located to the database.
                        self.db.update_kodiak_db_kodiak_port(kw.ipaddress, x)
                        return response[0]['triggered']

        return None

    def check_if_running(self, kw):
        response = self.get_module_capture_status(kw)
        # Checking for a bad response
        if not response or 'recording' not in response[0]:
            return False

        # Checking if the kodiak is streaming
        if response[0]['recording']:
            return False

        # return True if the kodiak is in a state we can stream to
        return True

    def start_auto_locate_port(self, kw):
        # self.login(kw)
        self.db.insert_kodiak_db_standard(kw.ipaddress, kw.key)

        # kw.username = KodiakGlobals.my_kodiak_user
        # kw.password = KodiakGlobals.my_kodiak_pass

        # if not self.login(kw):
        #     self.create_user(kw)
        #
        # if not self.login(kw):
        #     return False

        self.lock(kw, 'SB_LOCK')
        if not self.check_if_running(kw):
            return False

        # Start the kodiak, if it fails to start, return False
        if not self.start(kw, trigger=True):
            return False

        # Start the locate port in another thread...
        # Done this because we don't want to hold the current request thread!
        # thread = threading.Thread(target=self.locate_kodiak_port, args=(kw, '192.168.1.160', ))
        # thread.start()

        self.locate_kodiak_port(kw, '192.168.1.160')

        # If we get here, the kodiak has been started and we're attempting to locate the port
        return True


class KodiakWrapper:
    def __init__(self, kodiak_ip_address, username, password, private_key):
        self.password = password
        self.username = username
        self.key = private_key
        self.ipaddress = kodiak_ip_address



import logging
import json

from .RestFunctions import do_POST, do_GET
from .KodiakGlobals import KodiakGlobals
from .kodiak_endpoints import *


class KodiakSession:
    def __init__(self):
        """
        Kodiak session class is designed to control the login / authentication aspects of the Kodiak.
        Separated this from the main code so that program can store all these identifiers in isolated location.

        """
        pass

    def login(self, kw):
        """
        Sending a POST request to log in to the kodiak system

        :return: Dictionary if login worked, else None
        """

        if not kw.username:
            logging.warning('No username provided for login!')
            return None

        if not kw.password:
            logging.warning('No password provided for login!')
            return None

        # Message to send to endpoint
        message_to_send = {
            "username": kw.username,
            "password": kw.password
        }

        # Authentication header
        headers = {
            'x-api-key': kw.key
        }

        endpoint = KodiakGlobals.url_prefix + kw.ipaddress + endpoint_login

        # A post request to the API
        response = self.send_post_command(message_to_send, endpoint, headers)

        try:
            refresh_token = response['refresh_token']
            session_token = response['session_token']
        except KeyError:
            logging.warning(f'Session or lock key was missing from login post request! : {response}')
            return None

        return response

    # def refresh_login(self, kw, refresh_token, private_key):
    #     """
    #     Sending a post request to the refresh_login endpoint.
    #     Will try to get another 7 day session key.\
    #
    #     :param kw: Kodiak Wrapper
    #     :param refresh_token: Refresh token from the last log in
    #     :param private_key: Private API key from user
    #
    #     :return: Dictionary if login worked, else None
    #     """
    #
    #     # Message to send to endpoint
    #     message_to_send = {
    #         "token": refresh_token,
    #     }
    #
    #     # Authentication header
    #     headers = {
    #         "X-Auth-Token": private_key
    #     }
    #
    #     # Create the login endpoint
    #     endpoint = KodiakGlobals.url_prefix + kw.ipaddress + endpoint_refresh_login
    #
    #     # A post request to the API
    #     response = self.send_post_command(message_to_send, endpoint, headers)
    #
    #     try:
    #         refresh_token = response['refresh_token']
    #         session_token = response['session_token']
    #
    #     except KeyError:
    #         logging.warning(f'Session or lock key was missing from login post request! : {response}')
    #         # Return none indicating a bad response
    #         return None
    #
    #     # Return the response from the refresh
    #     return response

    def get_users(self, kw) -> str:
        """
        Send a GET request to the Kodiak get the current list of users attached

        :param kw: Kodiak wrapper containing credentials

        :return: Response from get users
        """
        headers = {
            'x-api-key': kw.key
        }
        endpoint = KodiakGlobals.url_prefix + kw.ipaddress + endpoint_users

        # Do the get request to lock endpoint
        response = self.send_get_request(endpoint, headers)

        return response

    def post_add_users(self, kw) -> str:
        """
        Send a POST request to the Kodiak add a new user

        :param kw: Kodiak wrapper containing credentials

        :return: true if user was added, else false
        """

        message_to_send = {
            "username": KodiakGlobals.my_kodiak_user,
            "password": KodiakGlobals.my_kodiak_pass,
            "groups": ["Administrators"] # This group should be default on all kodiaks
        }
        headers = {
            'x-api-key': kw.key
        }

        endpoint = KodiakGlobals.url_prefix + kw.ipaddress + endpoint_users

        # Do the get request to lock endpoint
        response = self.send_post_command(message_to_send, endpoint, headers)

        return response

    def get_lock_status(self, kw, session_token) -> str:
        """
        Send a GET request to the Kodiak to see its current lock status.

        :param kw: Kodiak wrapper containing credentials
        :param session_token: Login session token

        :return: Response from get lock stats request
        """

        # Header to include in POST request
        headers = {
            'x-api-key': kw.key
        }

        endpoint = KodiakGlobals.url_prefix + kw.ipaddress + endpoint_status

        # Do the get request to lock endpoint
        response = self.send_get_request(endpoint, headers)

        return response

    def lock(self, kw, lock_name, session_token):
        """
        Locking the system so we can control it without interference

        :param kw: Kodiak wrapper containing credentials
        :param lock_name: Name you wish to give the lock session
        :param session_token: Login session token

        :return: boolean for success
        """

        message_to_send = {
            "lock_name": lock_name,
            # "owner": kw.username,
            "force": True
        }
        headers = {
            # "X-Auth-Token": session_token
            'x-api-key': kw.key
        }

        endpoint = KodiakGlobals.url_prefix + kw.ipaddress + endpoint_lock

        try:
            response = self.send_post_command(message_to_send, endpoint, headers)

            if 'key' in response.keys():
                return response['key']

            return None

        except KeyError:
            logging.warning(f'No lock key was returned from lock post request! : {response}')
            return None

        except Exception as err:
            logging.error(err)
            return None

    def unlock(self, kw, lock_key, session_token):
        """
        Sending a post request to unlock the Kodiak device

        :param kw: Kodiak wrapper containing credentials
        :param lock_key: Key that locked the module.
        :param session_token: Login session token

        :return: False if unlock failed, true if success
        """

        # Creating the message to send to the endpoint
        message_to_send = {
            "key": lock_key
        }
        headers = {
            'x-api-key': kw.key
        }

        # Endpoint location
        endpoint = KodiakGlobals.url_prefix + kw.ipaddress + endpoint_unlock

        # A post request to the API
        response = self.send_post_command(message_to_send, endpoint, headers)\

        # TODO: Check what happens if this fails to unlock

        return True

    def send_post_command(self, command, endpoint, header):
        """
        Creating a post command wrapper to make teh json object and examine the response for HTTP errors
        :param command:
        :param endpoint:
        :param header:
        :return: Dict - Json data returned
        """
        # converting the msg to json
        json_object = json.dumps(command, indent=4)

        # Post the command
        response = do_POST(command, endpoint, header)

        # if not self._check_response(response):
        #     self._handle_invalid_response(response)

        json_data = json.loads(str(response.text))

        return json_data

    def send_get_request(self, endpoint, header):
        """
        Creating a wrapper for sending GET requests to examine the response return for HTTP errors
        :param endpoint:
        :param header:
        :return:
        """

        # Do get request
        response = do_GET(endpoint, header)

        # if not self._check_response(response):
        #     self._handle_invalid_response(response)

        json_data = json.loads(str(response.text))

        return json_data

    def _check_response(self, response):
        """
        Checking if the response from the module is valid.
        Will Warn if the response was invalid.

        :param response: Response object
        :return: Boolean - true if valid, else false
        """
        if 200 <= int(response.status_code) <= 229:
            return True

        logging.warning(response.text)

        return False

    def _handle_invalid_response(self, response):

        if int(response.status_code) == 401:
            # Indicates the user is not logged authenticated - a login / refresh token is required.
            if self.logged_in:
                self.refresh_required = True
            self.logged_in = False



import time
from zeroconf import ServiceBrowser, Zeroconf
import socket
from .KodiakGlobals import KodiakGlobals


class MyListener:
    def __init__(self):
        self.found_ip_addresses = []
    #
    # def remove_service(self, zeroconf, type, name):
    #     print(f"Service {name} removed")

    def add_service(self, zeroconf, type, name):
        """
        Overriding the add service method to get and add the kodiak IP addresses that were discovered over MDNs

        Args:
            zeroconf: ZeroConf service object
            type: Type you're looking for (kodiak's response)
            name: Name of the object

        Returns:
            Nothing
            But adds the found IPaddress to a list which can be accessed elsewhere
        """
        info = zeroconf.get_service_info(type, name)
        if info:
            # print(f"Service {name} added, service info: {info}")
            ip_addr = socket.inet_ntoa(info.addresses[0])
            # print(ip_addr)
            self.found_ip_addresses.append(ip_addr)

    def update_service(self):
        """
        Adding to stop the mdns warning message about this 'soon to be necessary' function.

        Returns:
            N/A
        """
        pass


def find_kodiak_ip_addresses(runtime=1):
    """
    Function to get the IPAddresses of the kodiaks discovered via MDNS

    Args:
        runtime: Time to wait for discovery

    Returns:
        List - List of IPAddresses (strings)

    """

    zeroconf = Zeroconf()
    listener = MyListener()

    browser = ServiceBrowser(zeroconf, KodiakGlobals.kodiak_mdns_locator, listener)

    try:
        time.sleep(1)
    except KeyboardInterrupt:
        pass
    finally:
        zeroconf.close()

    return listener.found_ip_addresses
    # print(listener.found_ip_addresses)

# find_kodiak_ip_addresses()


endpoint_stop = "/kodiak/v1/device/capture/stop"
endpoint_start = "/kodiak/v1/device/capture/start"

endpoint_capture_status = "/kodiak/v1/device/capture/status"
endpoint_status = "/kodiak/v1/status"


endpoint_users = "/kodiak/v1/users"

# endpoint_trigger_status = "/kodiak/v1/device/capture/trigger/status"

endpoint_login = "/kodiak/v1/login"
endpoint_refresh_login = "/kodiak/v1/login/refresh"

endpoint_lock = "/kodiak/v1/device/lock"
endpoint_unlock = "/kodiak/v1/device/unlock"

endpoint_lock_status = "/kodiak/v1/device/lock"



import logging
import sqlite3
import time


class SimpleDB:
    def __init__(self, db_name):
        self.database_name = db_name
        self.conn = None

        # self.delete_table()
        if not self.check_exists():
            self.create_table()

    def db_connect(self):
        self.conn = sqlite3.connect(self.database_name)

    def db_close(self):
        self.conn.close()

    def check_exists(self):
        """
        Simple method to check the database exists.

        :return: Boolean : True if exists else False
        """

        self.db_connect()

        cur = self.conn.cursor()

        try:
            cur.execute(f"SELECT * FROM {self.database_name}")

            # storing the data in a list
            data_list = cur.fetchall()

            return True

        except sqlite3.OperationalError:
            return False

        finally:
            self.db_close()

    def create_table(self):
        query = """
            CREATE TABLE IF NOT EXISTS kodiak_db (
                id TEXT PRIMARY KEY,
                username TEXT,
                password TEXT,
                api_key TEXT,
                session TEXT,
                session_time TEXT,
                refresh_token TEXT,
                kodiak_port TEXT,
                lock_key TEXT
            )
        """
        self.db_connect()
        cursor = self.conn.cursor()

        try:
            cursor.execute(query)
            self.conn.commit()
            if cursor.rowcount > 0:
                return True
            else:
                return False
        finally:
            self.db_close()


    def insert_kodiak_db_standard(self, ipaddress, key):
        query = "INSERT INTO kodiak_db (id, api_key) VALUES (?, ?)"
        self.db_connect()

        cursor = self.conn.cursor()

        try:
            cursor.execute(query, (ipaddress, key))
            self.conn.commit()


            if cursor.rowcount > 0:
                return True
            else:
                return False

            self.db_close()

        except sqlite3.IntegrityError as error:
            self.db_close()
            # the value we`re trying to update is already in the DB
            return self.update_kodiak_db(ipaddress, key)
        # finally:
        #     self.db_close()

    def get_kodiak_db(self, ipaddress):
        """
        Get the row from the DB with the specified Kodiak IP
        ORDER: 0 : id TEXT PRIMARY KEY,
               1 : username TEXT,
               2 : password TEXT,
               3 : api_key TEXT,
               4 : session TEXT,
               5 : session_time TEXT,
               6 : refresh_token TEXT,
               7 : kodiak_port TEXT,
               8 : lock_key TEXT

        :param ipaddress: IPaddress of the kodiak module

        :return: Return tuple or results if success else None
        """

        query = "SELECT * FROM kodiak_db WHERE id = ?"

        self.db_connect()

        cursor = self.conn.execute(query, (ipaddress,))
        result = cursor.fetchone()

        if not result:
            result = None

        self.db_close()

        return result

    def get_ip_addresses(self):

        """
        Get every IP address currently found that doesn't have an API Key
        ORDER: 0 : id TEXT PRIMARY KEY,
               1 : username TEXT,
               2 : password TEXT,
               3 : api_key TEXT,
               4 : session TEXT,
               5 : session_time TEXT,
               6 : refresh_token TEXT,
               7 : kodiak_port TEXT,
               8 : lock_key TEXT

        :param ipaddress: IPaddress of the kodiak module

        :return: Return tuple or results if success else None
        """

        query = "SELECT * FROM kodiak_db WHERE id = ?"

        self.db_connect()

        cursor = self.conn.execute(query, (ipaddress,))
        result = cursor.fetchone()

        if not result:
            result = None

        self.db_close()

        return result

    def update_kodiak_db(self, ipaddress, key):
        self.db_connect()

        query = "UPDATE kodiak_db SET api_key = ? WHERE id = ?"
        cursor = self.conn.cursor()

        try:
            cursor.execute(query, (key, ipaddress))
            self.conn.commit()

            if cursor.rowcount > 0:
                return True
            else:
                return False
        except Exception as err:
            logging.warning(err)
            return False

        finally:
            self.db_close()

    def update_kodiak_db_session_key(self, ipaddress, session_key, refresh_token):
        self.db_connect()

        query = "UPDATE kodiak_db SET session = ? WHERE id = ?"
        self.conn.execute(query, (session_key, ipaddress))
        query = "UPDATE kodiak_db SET session_time = ? WHERE id = ?"
        self.conn.execute(query, (time.time(), ipaddress))
        query = "UPDATE kodiak_db SET refresh_token = ? WHERE id = ?"
        self.conn.execute(query, (refresh_token, ipaddress))
        self.conn.commit()

        self.db_close()

    def update_kodiak_db_lock_key(self, ipaddress, lock_key):
        query = "UPDATE kodiak_db SET lock_key = ? WHERE id = ?"

        self.db_connect()
        cursor = self.conn.cursor()

        try:
            cursor.execute(query, (lock_key, ipaddress))
            self.conn.commit()

            if cursor.rowcount > 0:
                return True
            else:
                logging.warning("Failed to update lock_key to db")
                return False
        finally:
            self.db_close()

    def update_kodiak_db_kodiak_port(self, ipaddress, port):
        query = "UPDATE kodiak_db SET kodiak_port = ? WHERE id = ?"

        self.db_connect()

        cursor = self.conn.cursor()

        try:
            cursor.execute(query, (port, ipaddress))
            self.conn.commit()

            if cursor.rowcount > 0:
                return True
            else:
                return False
        finally:
            self.db_close()

    def update_kodiak_db_kodiak_user(self, ipaddress, username):
        query = "UPDATE kodiak_db SET username = ? WHERE id = ?" #password

        self.db_connect()

        cursor = self.conn.cursor()

        try:
            cursor.execute(query, (username, ipaddress))
            self.conn.commit()

            if cursor.rowcount > 0:
                return True
            else:
                return False
        finally:
            self.db_close()

    def update_kodiak_db_kodiak_password(self, ipaddress, password):
        query = "UPDATE kodiak_db SET password = ? WHERE id = ?" #password

        self.db_connect()

        cursor = self.conn.cursor()

        try:
            cursor.execute(query, (password, ipaddress))
            self.conn.commit()

            if cursor.rowcount > 0:
                return True
            else:
                return False
        finally:
            self.db_close()

    def remove_kodiak_from_db(self, ipaddress):
        query = "DELETE FROM kodiak_db WHERE id = ?"

        self.db_connect()

        self.conn.execute(query, (ipaddress,))
        self.conn.commit()

        self.db_close()

        return True

    def delete_table(self):
        query = "DROP TABLE IF EXISTS kodiak_db"

        self.db_connect()

        cursor = self.conn.cursor()

        cursor.execute(query)
        self.conn.commit()

        try:
            if cursor.rowcount > 0:
                return True
            else:
                return False
        finally:
            self.db_close()


# Example usage
if __name__ == "__main__":
    db = SimpleDB("mykodiak_dbbase.db")
    ipaddress = '127.0.0.1'
    user = "matt"
    password = 'holsey'
    key = 'somekey'

    db.db_connect()

    db.delete_table()

    db.create_table()

    db.insert_kodiak_db_standard(ipaddress, key)

    print(f"Value of '{ipaddress}':", db.get_kodiak_db("127.0.0.1"))

    db.update_kodiak_db(ipaddress, key + "2")

    print(f"Updated value of '{ipaddress}':", db.get_kodiak_db(ipaddress))

    db.remove_kodiak_from_db(ipaddress)

    print(f'Removed value of {ipaddress} from table')

    print(f'Checking it was removed: {db.get_kodiak_db(ipaddress)}')






import pytest
from kodiak_testing.SanBlazeWebServer.SBFlask import create_app


@pytest.fixture()
def app():
    app = create_app()
    app.config.update({
        "TESTING": True,
    })

    # other setup can go here

    yield app

    # clean up / reset resources here


@pytest.fixture()
def client(app):
    return app.test_client()


@pytest.fixture()
def runner(app):
    return app.test_cli_runner()





class KodiakGlobals:
    url_prefix = 'https://'
    last_response = []
    lock_name = "sanblaze_lock"
    database_name = "sanblaze_db"

    my_kodiak_ip = "192.168.1.247"
    my_kodiak_user = "mholsey"
    # my_kodiak_user = "sanblaze"
    my_kodiak_pass = "sb"

    kodiak_mdns_locator = '_g5_busxpert._tcp.local.'



from setuptools import setup

setup(name='SBKodiak',
      version='1.1.9.dev2',
      description='SanBlaze Python control of Kodiak Modules',
      author='Matt Holsey',
      author_email='mholsey@sanblaze.com',
      url='https://www.sanblaze.com/',
      license='SanBlaze Technology ltd',
      packages=['SBKodiak', 
				'SBKodiak.docs',
				'SBKodiak.tests'],
	  install_requires=[
			    'paramiko',
			    'requests',
				'urllib3',
                'zeroconf',
		  ],
      include_package_data=True,
      zip_safe=False)


login_response_good = {
    'refresh_token': 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJYLXJlZnJlc2giOnRydWUsIlgtdXNlcm5hbWUiOiJhZG1pbiIsImV4cCI6MTY5MzkyNDQyNCwiaWF0IjoxNjkzMzE5NjI0fQ.0tHMvgn7C1Tpd9udlYIEdjE61ZXC90uVTNvPdgbi4lQ',
    'session_token': "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJYLWRpc3BsYXluYW1lIjoiQWRtaW5pc3RyYXRvciIsIlgtcHJpdmlsZWdlcyI6WyJMb2dpbiIsIkNvbmZpZ3VyZVNlbGYiLCJSZWFkTWVkaWEiLCJDb25maWd1cmVNZWRpYSIsIlJlYWRVc2VyIiwiQ29uZmlndXJlVXNlciIsIkNvbmZpZ3VyZVN5c3RlbSIsIkNvbmZpZ3VyZUZpcm13YXJlIiwiQ29uZmlndXJlQ2FwdHVyZSIsIlJlYWREZXZpY2UiLCJSZWFkVHJhY2UiLCJDb25maWd1cmVUcmFjZSIsIkNvbmZpZ3VyZVN0b3JlIl0sIlgtdXNlcm5hbWUiOiJhZG1pbiIsImV4cCI6MTY5MzMyMTQyNCwiaWF0IjoxNjkzMzE5NjI0fQ.gQP-ZRFpEBKKXHpsF9lRe3UQvG0CRmylsbA-IH9EI6Y"

}

login_response_bad = {
  "error": {
    "code": "InvalidCredentials",
    "data": None,
    "message": "The provided credentials are invalid"
  }
}


refresh_login_responses = {
    'refresh_token': 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJYLXJlZnJlc2giOnRydWUsIlgtdXNlcm5hbWUiOiJhZG1pbiIsImV4cCI6MTY5MzkyNDQyNCwiaWF0IjoxNjkzMzE5NjI0fQ.0tHMvgn7C1Tpd9udlYIEdjE61ZXC90uVTNvPdgbi4lQ',
    'session_token': 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJYLWRpc3BsYXluYW1lIjoiQWRtaW5pc3RyYXRvciIsIlgtcHJpdmlsZWdlcyI6WyJMb2dpbiIsIkNvbmZpZ3VyZVNlbGYiLCJSZWFkTWVkaWEiLCJDb25maWd1cmVNZWRpYSIsIlJlYWRVc2VyIiwiQ29uZmlndXJlVXNlciIsIkNvbmZpZ3VyZVN5c3RlbSIsIkNvbmZpZ3VyZUZpcm13YXJlIiwiQ29uZmlndXJlQ2FwdHVyZSIsIlJlYWREZXZpY2UiLCJSZWFkVHJhY2UiLCJDb25maWd1cmVUcmFjZSIsIkNvbmZpZ3VyZVN0b3JlIl0sIlgtdXNlcm5hbWUiOiJhZG1pbiIsImV4cCI6MTY5MzMyMTYxNywiaWF0IjoxNjkzMzE5ODE3fQ.seRFatv_Rq6oSRaKiosvWP64Z58IVWADReVAdSTdyZ4'
}

refresh_login_response_bad = {
  "error": {
    "code": "UnhandledException",
    "data": {
      "exception": "signature format is incorrect"
    },
    "message": "An exception occurred that was not handled"
  }
}


lock_response_bad = {"error":
                         {"code": "LockError",
                          "data":
                              {"accessed": "2023-08-28T19:50:37+0000",
                               "acquired": "2023-08-28T19:50:37+0000",
                               "lock_owner": "Administrator"},
                          "message": "The analyser is currently locked"
                          }
                     }

lock_response_bad2 = {"error":
                          {'code': 'LockError',
                           'data':
                               {'accessed': '2023-08-29T15:27:19+0000',
                                'acquired': '2023-08-29T15:27:19+0000',
                                'lock_owner': 'Live Trace'},
                           'message': 'The analyser is currently locked'
                           }
                      }

lock_response_good = {"accessed": "2023-08-29T15:37:43+0000",
                      "acquired": "2023-08-29T15:37:43+0000",
                      "id": "H+DBL3Lj",
                      "key": "rsaHohL9",
                      "lock_name": "Administrator",
                      "owner": "Administrator"
                      }

lock_status_good = {'accessed': '2023-08-29T15:37:43+0000',
                    'acquired': '2023-08-29T15:37:43+0000',
                    'id': 'H+DBL3Lj',
                    'lock_name': 'Administrator',
                    'owner': 'Administrator'}


unlock_response_good = {
    "accessed": None,
    "acquired": None,
    "id": "",
    "lock_name": "",
    "owner": ""
}

unlock_response_bad = {
  "error": {
    "code": "LockError",
    "data": {
      "accessed": "2023-08-31T17:08:11+0000",
      "acquired": "2023-08-31T17:08:11+0000",
      "lock_owner": "Administrator"
    },
    "message": "The analyser is currently locked"
  }
}

start_response_good = {
    [{"buffer": {"limit": 75497328, "max_buffer_size": 77309411328, "trigger_point": None, "trigger_position": 100,
                 "used": 144, "wrapped": False},
      "current": {"almp": 0, "cachemem": 0, "cxl": False, "dllps": 22865, "eidle": False, "errors": 0, "io": 0,
                  "speed": 2, "tlps": 3, "training": 0, "width": 2},
      "last_second": {"almp": 0, "cachemem": 0, "cxl": 0, "dllps": 91442, "eidle": 0, "errors": 0, "io": 0, "tlps": 3,
                      "training": 0}, "latest_ts": 0, "name": "dn.0", "overflowed": False, "recording": True,
      "trigger_ts": 18446744073709551615, "triggered": False, "type": "data"}, {
         "buffer": {"limit": 75497328, "max_buffer_size": 77309411328, "trigger_point": None, "trigger_position": 100,
                    "used": 432, "wrapped": False},
         "current": {"almp": 0, "cachemem": 0, "cxl": False, "dllps": 33277, "eidle": False, "errors": 0, "io": 0,
                     "speed": 2, "tlps": 3, "training": 0, "width": 2},
         "last_second": {"almp": 0, "cachemem": 0, "cxl": 0, "dllps": 133093, "eidle": 0, "errors": 0, "io": 0,
                         "tlps": 3, "training": 0}, "latest_ts": 0, "name": "up.0", "overflowed": False,
         "recording": True, "trigger_ts": 18446744073709551615, "triggered": False, "type": "data"},
     {"buffer": {"limit": 1073741824, "max_buffer_size": 1073741824, "used": 0, "wrapped": False}, "current": {
         "signals": [1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1]},
      "name": "sideband.0", "type": "sideband"}, {
         "buffer": {"limit": 1073741824, "max_buffer_size": 1073741824, "trigger_point": None, "trigger_position": 100,
                    "used": 0, "wrapped": False}, "current": {"nacks": 0, "reads": 8, "timeouts": 0, "writes": 6},
         "last_second": {"nacks": 0, "reads": 16, "timeouts": 0, "writes": 12}, "latest_ts": 0, "name": "smbus.0",
         "recording": False, "trigger_ts": 18446744073709551116, "triggered": False, "type": "smbus"}],
    [{'buffer': {'limit': 75497328, 'max_buffer_size': 77309411328, 'trigger_point': None, 'trigger_position': 100,
                 'used': 144, 'wrapped': False},
      'current': {'almp': 0, 'cachemem': 0, 'cxl': False, 'dllps': 22865, 'eidle': False, 'errors': 0, 'io': 0,
                  'speed': 2, 'tlps': 3, 'training': 0, 'width': 2},
      'last_second': {'almp': 0, 'cachemem': 0, 'cxl': 0, 'dllps': 91442, 'eidle': 0, 'errors': 0, 'io': 0, 'tlps': 3,
                      'training': 0}, 'latest_ts': 0, 'name': 'dn.0', 'overflowed': False, 'recording': True,
      'trigger_ts': 18446744073709551615, 'triggered': False, 'type': 'data'}, {
        'buffer': {'limit': 75497328, 'max_buffer_size': 77309411328, 'trigger_point': None, 'trigger_position': 100,
                   'used': 432, 'wrapped': False},
        'current': {'almp': 0, 'cachemem': 0, 'cxl': False, 'dllps': 33277, 'eidle': False, 'errors': 0, 'io': 0,
                    'speed': 2, 'tlps': 3, 'training': 0, 'width': 2},
        'last_second': {'almp': 0, 'cachemem': 0, 'cxl': 0, 'dllps': 133093, 'eidle': 0, 'errors': 0, 'io': 0,
                        'tlps': 3, 'training': 0}, 'latest_ts': 0, 'name': 'up.0', 'overflowed': False,
        'recording': True, 'trigger_ts': 18446744073709551615, 'triggered': False, 'type': 'data'}, {
        'buffer': {'limit': 1073741824, 'max_buffer_size': 1073741824, 'used': 0, 'wrapped': False}, 'current': {
            'signals': [1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
                        1]}, 'name': 'sideband.0', 'type': 'sideband'}, {
        'buffer': {'limit': 1073741824, 'max_buffer_size': 1073741824, 'trigger_point': None, 'trigger_position': 100,
                   'used': 0, 'wrapped': False}, 'current': {'nacks': 0, 'reads': 8, 'timeouts': 0, 'writes': 6},
        'last_second': {'nacks': 0, 'reads': 16, 'timeouts': 0, 'writes': 12}, 'latest_ts': 0, 'name': 'smbus.0',
        'recording': False, 'trigger_ts': 18446744073709551116, 'triggered': False, 'type': 'smbus'}]
}

start_response_good2 = {
    [{"buffer": {"limit": 75497328, "max_buffer_size": 77309411328, "trigger_point": None, "trigger_position": 100,
                 "used": 0, "wrapped": False},
      "current": {"almp": 0, "cachemem": 0, "cxl": False, "dllps": 0, "eidle": True, "errors": 0, "io": 0, "speed": 0,
                  "tlps": 0, "training": 0, "width": 0},
      "last_second": {"almp": 0, "cachemem": 0, "cxl": 0, "dllps": 0, "eidle": 4, "errors": 0, "io": 0, "tlps": 0,
                      "training": 0}, "latest_ts": 0, "name": "dn.0", "overflowed": False, "recording": True,
      "trigger_ts": 18446744073709551615, "triggered": False, "type": "data"}, {
         "buffer": {"limit": 75497328, "max_buffer_size": 77309411328, "trigger_point": None, "trigger_position": 100,
                    "used": 0, "wrapped": False},
         "current": {"almp": 0, "cachemem": 0, "cxl": False, "dllps": 0, "eidle": True, "errors": 0, "io": 0,
                     "speed": 2, "tlps": 0, "training": 0, "width": 2},
         "last_second": {"almp": 0, "cachemem": 0, "cxl": 0, "dllps": 0, "eidle": 4, "errors": 0, "io": 0, "tlps": 0,
                         "training": 0}, "latest_ts": 0, "name": "up.0", "overflowed": False, "recording": True,
         "trigger_ts": 18446744073709551615, "triggered": False, "type": "data"},
     {"buffer": {"limit": 1073741824, "max_buffer_size": 1073741824, "used": 0, "wrapped": False}, "current": {
         "signals": [1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1]},
      "name": "sideband.0", "type": "sideband"}, {
         "buffer": {"limit": 1073741824, "max_buffer_size": 1073741824, "trigger_point": None, "trigger_position": 100,
                    "used": 0, "wrapped": False}, "current": {"nacks": 0, "reads": 0, "timeouts": 0, "writes": 0},
         "last_second": {"nacks": 0, "reads": 0, "timeouts": 0, "writes": 0}, "latest_ts": 0, "name": "smbus.0",
         "recording": False, "trigger_ts": 18446744073709551116, "triggered": False, "type": "smbus"}],
    [{'buffer': {'limit': 75497328, 'max_buffer_size': 77309411328, 'trigger_point': None, 'trigger_position': 100,
                 'used': 0, 'wrapped': False},
      'current': {'almp': 0, 'cachemem': 0, 'cxl': False, 'dllps': 0, 'eidle': True, 'errors': 0, 'io': 0, 'speed': 0,
                  'tlps': 0, 'training': 0, 'width': 0},
      'last_second': {'almp': 0, 'cachemem': 0, 'cxl': 0, 'dllps': 0, 'eidle': 4, 'errors': 0, 'io': 0, 'tlps': 0,
                      'training': 0}, 'latest_ts': 0, 'name': 'dn.0', 'overflowed': False, 'recording': True,
      'trigger_ts': 18446744073709551615, 'triggered': False, 'type': 'data'}, {
         'buffer': {'limit': 75497328, 'max_buffer_size': 77309411328, 'trigger_point': None, 'trigger_position': 100,
                    'used': 0, 'wrapped': False},
         'current': {'almp': 0, 'cachemem': 0, 'cxl': False, 'dllps': 0, 'eidle': True, 'errors': 0, 'io': 0,
                     'speed': 2, 'tlps': 0, 'training': 0, 'width': 2},
         'last_second': {'almp': 0, 'cachemem': 0, 'cxl': 0, 'dllps': 0, 'eidle': 4, 'errors': 0, 'io': 0, 'tlps': 0,
                         'training': 0}, 'latest_ts': 0, 'name': 'up.0', 'overflowed': False, 'recording': True,
         'trigger_ts': 18446744073709551615, 'triggered': False, 'type': 'data'},
     {'buffer': {'limit': 1073741824, 'max_buffer_size': 1073741824, 'used': 0, 'wrapped': False}, 'current': {
         'signals': [1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1]},
      'name': 'sideband.0', 'type': 'sideband'}, {
         'buffer': {'limit': 1073741824, 'max_buffer_size': 1073741824, 'trigger_point': None, 'trigger_position': 100,
                    'used': 0, 'wrapped': False}, 'current': {'nacks': 0, 'reads': 0, 'timeouts': 0, 'writes': 0},
         'last_second': {'nacks': 0, 'reads': 0, 'timeouts': 0, 'writes': 0}, 'latest_ts': 0, 'name': 'smbus.0',
         'recording': False, 'trigger_ts': 18446744073709551116, 'triggered': False, 'type': 'smbus'}]
}

start_response_bad = {
  "error": {
    "code": "MissingParameter",
    "data": {
      "parameter": "lock_key"
    },
    "message": "A required parameter was not present in the request"
  }
}

start_hang_response = '<!DOCTYPE html>\n<html>\n<head>\n<title>Error</title>\n<style>\n    body {\n        width: 35em;\n        margin: 0 auto;\n        font-family: Tahoma, Verdana, Arial, sans-serif;\n    }\n</style>\n</head>\n<body>\n<h1>An error occurred.</h1>\n<p>Sorry, the page you are looking for is currently unavailable.<br/>\nPlease try again later.</p>\n<p>If you are the system administrator of this resource then you should check\nthe error log for details.</p>\n<p><em>Faithfully yours, nginx.</em></p>\n</body>\n</html>\n'


stop_response_bad = {
  "error": {
    "code": "LockError",
    "data": {
      "accessed": "2023-08-31T17:27:32+0000",
      "acquired": "2023-08-31T17:08:11+0000",
      "lock_owner": "Administrator"
    },
    "message": "The provided lock key does not match the lock"
  }
}


get_users_good = {
  "count": 5,
  "members": [
    {
      "uri": "/kodiak/v1/users/admin"
    },
    {
      "uri": "/kodiak/v1/users/davidn"
    },
    {
      "uri": "/kodiak/v1/users/esse quis velit"
    },
    {
      "uri": "/kodiak/v1/users/mholsey"
    },
    {
      "uri": "/kodiak/v1/users/shynes"
    }
  ]
}


import os
import time

import paramiko

from kodiak_testing.KodiakAPI.KodiakControl import KodiakControlClass, KodiakWrapper
from kodiak_testing.KodiakAPI.KodiakGlobals import KodiakGlobals


def do_main():
    print('Running test through system')

    kw = KodiakWrapper("192.168.1.247", "mholsey", "", "gThV9h4LQmmSiYFNTcjBuvoRPvEC3LpW")

    print('Making a Kodiak Control object')
    my_kodiak = KodiakControlClass()

    print(f'Logging into kodiak using: {KodiakGlobals.my_kodiak_user}, {KodiakGlobals.my_kodiak_pass}')
    my_kodiak.login(kw)
    print('Logged in!') if my_kodiak.kodiak_session.logged_in else print('Login Variable not set!')

    print('Checking if we can get a refresh of the login')
    my_kodiak.refresh(kw)

    print('Locking the Module')
    if not my_kodiak.lock(kw):
        print("Failed to lock the system!")
        print(KodiakGlobals.last_response[-1])
        return

    print('Checking lock status')
    print(my_kodiak.get_lock_status(kw))

    # x = input('Is the lock correct?')
    # default_value = "n"
    # if x is default_value:
    #     return

    print('Starting the trace')
    print(my_kodiak.start(kw, trigger=True))

    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh.connect('192.168.1.160', 22, username='root', password='sb')

    for x in range(16):
        if len(str(x)) < 2:
            x = "0" + str(x)

        print(f'sending a trigger to 1{x}')
        os_cmd = f'sbecho trigger=1{x} > /proc/vlun/nvme'
        print(f'Sending os command : {os_cmd} : to IP : 192.168.1.160')

        ssh_stdin, ssh_stdout, ssh_stderr = ssh.exec_command(os_cmd)
        print(ssh_stdin)
        print(ssh_stdout)
        print(ssh_stderr)
        # print(os.system(f'plink -ssh root@192.168.1.160 -pw sb "{os_cmd}"'))

        time.sleep(0.05)

        print('checking if the module triggered')
        response = my_kodiak.get_module_capture_status(kw)

        print(response)

        if response:
            if response[0]['triggered']:
                print(response[0]['triggered'])
                print(f"MODULE WAS TRIGGERED ON PORT: 1{x}")
                break
    else:
        print("Did not trigger kodiak on any port, are you sure the trigger is correct and module attached?")
        # return

    print("stopping kodiak")
    print(my_kodiak.stop(kw))

    print('Unlocking Module')
    my_kodiak.unlock(kw)

    print('Checking the module is unlocked')
    print(my_kodiak.get_lock_status(kw))


if __name__ == "__main__":
    do_main()



"""
import os
import time

import paramiko

from kodiak_testing.KodiakAPI.KodiakControl import KodiakControlClass
from kodiak_testing.KodiakAPI.KodiakGlobals import KodiakGlobals


def do_main():
    print('Running test through system')

    print('Making a Kodiak Control object')
    my_kodiak = KodiakControlClass(kodiak_ip_address=KodiakGlobals.my_kodiak_ip, private_key="gThV9h4LQmmSiYFNTcjBuvoRPvEC3LpW")

    print(f'Logging into kodiak using: {KodiakGlobals.my_kodiak_user}, {KodiakGlobals.my_kodiak_pass}')
    my_kodiak.login(KodiakGlobals.my_kodiak_user, KodiakGlobals.my_kodiak_pass)
    print('Logged in!') if my_kodiak.kodiak_session.logged_in else print('Login Variable not set!')

    print('Checking if we can get a refresh of the login')
    my_kodiak.refresh()

    print('Locking the Module')
    if not my_kodiak.lock():
        print("Failed to lock the system!")
        print(KodiakGlobals.last_response[-1])
        return

    print('Checking lock status')
    print(my_kodiak.get_lock_status())

    # x = input('Is the lock correct?')
    # default_value = "n"
    # if x is default_value:
    #     return

    print('Starting the trace')
    print(my_kodiak.start(trigger=True))

    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh.connect('192.168.1.160', 22, username='root', password='sb')

    for x in range(16):
        if len(str(x)) < 2:
            x = "0" + str(x)

        print(f'sending a trigger to 1{x}')
        os_cmd = f'sbecho trigger=1{x} > /proc/vlun/nvme'
        print(f'Sending os command : {os_cmd} : to IP : 192.168.1.160')

        ssh_stdin, ssh_stdout, ssh_stderr = ssh.exec_command(os_cmd)
        print(ssh_stdin)
        print(ssh_stdout)
        print(ssh_stderr)
        # print(os.system(f'plink -ssh root@192.168.1.160 -pw sb "{os_cmd}"'))

        time.sleep(0.05)

        print('checking if the module triggered')
        response = my_kodiak.get_module_status()

        print(response)

        if response:
            if response[0]['triggered']:
                print(response[0]['triggered'])
                print(f"MODULE WAS TRIGGERED ON PORT: 1{x}")
                break
    else:
        print("Did not trigger kodiak on any port, are you sure the trigger is correct and module attached?")
        # return


    print("stopping kodiak")
    print(my_kodiak.stop())

    print('Unlocking Module')
    my_kodiak.unlock()


    print('Checking the module is unlocked')
    print(my_kodiak.get_lock_status())



if __name__ == "__main__":
    do_main()"""


from test_app import *

def test_request_example(client):
    response = client.get("/posts")
    assert b"<h2>Hello, World!</h2>" in response.data


# def test_json_data(client):
#     response = client.post("/graphql",
#                            json={ "query": """
#                                             query User($id: String!) {
#                                                 user(id: $id) {
#                                                     name
#                                                     theme
#                                                     picture_url
#                                                 }
#                                             }
#                                         """,
#                                     "variables"={"id": 2}
#                 })
#     assert response.json["data"]["user"]["name"] == "Flask"


import requests
import logging
import urllib3

from .KodiakGlobals import KodiakGlobals


# Disabling the certification warnings - Without this, every HTTPS request to kodiak will display a logging.warning
urllib3.disable_warnings()


def do_POST(json_msg, endpoint_url, header={}):
    """
    Common endpoint to POST the JSON to an endpoint

    :param json_msg: Json formatted message to send
    :param endpoint_url: URL to post to ( XXX.XXX.XXX.XXX/end/point )
    :param header: Header to include in request (optional)

    :return: Whatever is returned from the post request
    """

    print(f"DOING POST: {repr(json_msg)} to {endpoint_url} ")

    # Common requests python post
    response = requests.post(endpoint_url, json=json_msg, verify=False, headers=header)

    # Add the response to the list of responses - Debugging
    KodiakGlobals.last_response.append(repr(response.text))

    # log the response
    logging.debug(repr(response.text))
    logging.debug(response)

    print(repr(response.text))

    # return RAW response object
    return response


def do_GET(endpoint_url: str, header={}):
    """
    Common endpoint to GET response from an endpoint

    :param header: Optional header to send
    :param endpoint_url: URL to get to ( XXX.XXX.XXX.XXX/end/point )

    :return: Whatever is returned from the get request
    """

    # Common requests python post
    response = requests.get(endpoint_url, verify=False, headers=header)

    # Add the response to the list of responses - Debugging
    KodiakGlobals.last_response.append(repr(response.text))

    # Print the response
    logging.debug(response.text)

    # Returning raw response object
    return response

